import os
from hydra.cloud.abstract_platform import AbstractPlatform

class FastLocalPlatform(AbstractPlatform):
    def __init__(self, model_path, options):
        super().__init__(model_path, options)

    def train(self):
        os.system(" ".join([self.options, 'python3', self.model_path]))
        return 0

    def serve(self):
        pass



MODEL_PATH_DEFAULT = "train.py"
CLOUD_DEFAULT = "local"
CPU_COUNT_DEFAULT = 16
MEMORY_SIZE_DEFAULT = 8
GPU_COUNT_DEFAULT = 0
GPU_TYPE_DEFAULT = 'NVIDIA_TESLA_P4'
REGION_DEFAULT = 'us-west2'
IMAGE_TAG_DEFAULT = ''
IMAGE_URL_DEFAULT = ''
OPTIONS_DEFAULT = {}



import os
import json
from hydra.cloud.abstract_platform import AbstractPlatform

class GoogleCloudPlatform(AbstractPlatform):

    CPU_COST_PER_HOUR = 0.021811
    MEMORY_COST_PER_HOUR = 0.002923

    def __init__(
        self,
        model_path,
        git_url,
        commit_sha,
        github_token,
        cpu,
        memory,
        gpu_type,
        gpu_count,
        image_tag,
        image_url,
        options,
        region):

        self.git_url = git_url
        self.commit_sha = commit_sha
        self.github_token = github_token

        self.image_tag = image_tag
        self.image_url = image_url

        self.cpu = cpu
        self.memory = memory
        self.gpu_type = gpu_type
        self.gpu_count = str(gpu_count)

        self.region = region

        self.script_path = os.path.join(os.path.dirname(__file__), '../docker/gcp_execution.sh')
        self.machines_json = os.path.join(os.path.dirname(__file__), '../resources/gcp_machines.json')

        self.machine_type = self.find_machine()

        super().__init__(model_path, options)


    def find_machine(self):
        machines = json.load(open(self.machines_json))

        optimal_machine = {'machine_type': None, 'price': float('inf')}

        for machine in machines:
            machine['cpu_count'] = float(machine['cpu_count'])
            machine['memory'] = float(machine['memory'])

            if machine['cpu_count'] >= self.cpu and machine['memory'] >= self.memory:
                price = GoogleCloudPlatform.CPU_COST_PER_HOUR * machine['cpu_count'] + \
                    GoogleCloudPlatform.MEMORY_COST_PER_HOUR * machine['memory']

                if price < optimal_machine['price']:
                    optimal_machine['machine_type'] = machine['machine_type']
                    optimal_machine['price'] = price

        if optimal_machine['machine_type'] is None:
            raise Exception("Machine type satisfying {} vCPU and {} GB memory is not found!".format(self.cpu, self.memory))
        else:
            return optimal_machine['machine_type']


    def train(self):
        command = ['sh', self.script_path, '-g', self.git_url, '-c', self.commit_sha,
            '-o', self.github_token, '-m', self.model_path, '-r', self.region,
            '-t', self.image_tag, '-u', self.image_url, '-a', self.gpu_count, '-y', self.gpu_type,
            '-n', self.machine_type, '-p', self.options]

        self.run_command(command)
        return 0


    def serve(self):
        pass



import os
import json
import uuid
import boto3
from hydra.cloud.abstract_platform import AbstractPlatform

class AWSPlatform(AbstractPlatform):

    def __init__(
            self,
            model_path,
            project_name,
            git_url,
            commit_sha,
            github_token,
            cpu,
            memory,
            gpu_count,
            image_tag,
            image_url,
            options,
            region):

        self.project_name = project_name
        self.git_url = git_url
        self.commit_sha = commit_sha
        self.github_token = github_token

        self.image_tag = image_tag
        self.image_url = image_url

        self.cpu = cpu
        self.memory = memory
        self.gpu_count = str(gpu_count)

        self.region = region

        self.batch = boto3.client(
            service_name = 'batch',
            region_name='us-east-1',
            endpoint_url='https://batch.us-east-1.amazonaws.com'
        )

        super().__init__(model_path, options)
        self.options['HYDRA_GIT_URL'] = self.git_url
        self.options['HYDRA_COMMIT_SHA'] = self.commit_sha
        self.options['HYDRA_OAUTH_TOKEN'] = self.github_token
        self.options['HYDRA_MODEL_PATH'] = self.model_path
        self.options['HYDRA_PLATFORM'] = 'aws'

    def train(self):
        # create job definition
        jobName = f"{self.project_name}_{uuid.uuid1()}"
        jobDefName = f"job-def-{jobName}"
        jobQueue = 'hydra-ml-queue'
        environment_list = []

        for k, v in self.options.items():
            environment_list.append(
                {
                    "name": k,
                    "value": str(v)
                }
            )

        resp = self.batch.register_job_definition(
            jobDefinitionName=jobDefName,
            type='container',
            containerProperties={
                'image': '823217009914.dkr.ecr.us-east-1.amazonaws.com/hydra:master',
                'vcpus': self.cpu,
                'memory': self.memory*1000,
                'privileged': True,
                'environment': environment_list
            }
        )
        print(resp)
        submitJobResponse = self.batch.submit_job(
            jobName=jobName,
            jobQueue=jobQueue,
            jobDefinition=jobDefName
        )

        jobId = submitJobResponse['jobId']
        print('Submitted job [%s - %s] to the job queue [%s]' % (jobName, jobId, jobQueue))







from itertools import product


def inflate_options(dictionary_list):
    dictionary_list_inflated = []

    for dictionary in dictionary_list:
        dictionary_list_inflated += dict_product(dictionary)

    return dictionary_list_inflated


def dict_product(dictionary):
    """ Uppack {"name": "cool-exp", "lr": [0.01, 0.001]} to list of cartesian products
    [{"name": "cool-exp", "lr": 0.01}, {"name": "cool-exp", "lr": 0.001}]
    """
    products = []

    list_dict = {}
    for key, values in dictionary.items():
        if not isinstance(values, list):
            list_dict[key] = [values]
        else:
            list_dict[key] = values

    for values in product(*list_dict.values()):
        products.append(dict(zip(list_dict.keys(), values)))

    return products


def dict_to_string(packet):
    """ Convert dictionary {"epoch": 10, "lr": 0.01} to string "epoch=10 lr=0.01"
    """
    params = ""

    for key, value in packet.items():
        params += key + "=" + str(value) + " "

    return params.strip()



import pytest
from hydra.cli import *
from click.testing import CliRunner

TEST_YAML_PATH = "tests/resources/test.yaml"

MODEL_PATH = "d3bug.py"
REPO_URL = "https://georgian.io/"
COMMIT_SHA = "m1rr0r1ng"
FILE_PATH = "ones/and/zer0es"
GITHUB_TOKEN =  "Georgian"
JSON_OPTIONS = "{'epoch': 88}"

CPU = 8
MEMORY = 16
GPU_TYPE = 'NVIDIA Tesla P4'
GPU_COUNT = 1
IMAGE_TAG = "default"
REGION = "us-west2"
OPTIONS = "epoch=88 lr=0.01"
SCRIPT_PATH = "camp/flog/gnaw"
MACHINE_NAME = "macbook-pro"

def test_train_local_cil(mocker):
    mocker.patch(
        "hydra.cli.os.path.isfile",
        return_value=False
    )
    mocker.patch(
        "hydra.cli.check_repo",
        return_value=(REPO_URL, COMMIT_SHA)
    )
    mocker.patch(
        "hydra.cli.json.loads",
    )
    mocker.patch(
        'hydra.cli.inflate_options',
        return_value=[JSON_OPTIONS]
    )
    mocker.patch(
        'hydra.cli.dict_to_string',
        return_value=OPTIONS
    )
    mocker.patch(
        'hydra.cli.LocalPlatform.__init__',
        return_value=None
    )
    mocker.patch(
        'hydra.cli.LocalPlatform.train',
    )

    runner = CliRunner()
    result = runner.invoke(train,
        ['--model_path', MODEL_PATH,
         '--cloud', 'local',
         '--github_token', GITHUB_TOKEN,
         '--options', JSON_OPTIONS])

    LocalPlatform.__init__.assert_called_once_with(
        model_path=MODEL_PATH,
        options=OPTIONS,
        git_url=REPO_URL,
        commit_sha=COMMIT_SHA,
        github_token=GITHUB_TOKEN,
    )

    LocalPlatform.train.assert_called_once_with()

    assert result.exit_code == 0



def test_train_google_cloud_cli(mocker):
    mocker.patch(
        "hydra.cli.os.path.isfile",
        return_value=False
    )
    mocker.patch(
        "hydra.cli.check_repo",
        return_value=(REPO_URL, COMMIT_SHA)
    )
    mocker.patch(
        "hydra.cli.json.loads",
    )
    mocker.patch(
        'hydra.cli.inflate_options',
        return_value=[JSON_OPTIONS]
    )
    mocker.patch(
        'hydra.cli.dict_to_string',
        return_value=OPTIONS
    )
    mocker.patch(
        'hydra.cli.GoogleCloudPlatform.__init__',
        return_value=None
    )
    mocker.patch(
        'hydra.cli.GoogleCloudPlatform.train'
    )

    runner = CliRunner()
    result = runner.invoke(train,
        ['--model_path', MODEL_PATH,
         '--cloud', 'gcp',
         '--github_token', GITHUB_TOKEN,
         '--cpu_count', CPU,
         '--memory_size', MEMORY,
         '--gpu_count', GPU_COUNT,
         '--gpu_type', GPU_TYPE,
         '--region', REGION,
         '--image_tag', IMAGE_TAG,
         '--options', JSON_OPTIONS])

    GoogleCloudPlatform.__init__.assert_called_once_with(
        model_path=MODEL_PATH,
        github_token=GITHUB_TOKEN,
        cpu=CPU,
        memory=MEMORY,
        gpu_count=GPU_COUNT,
        gpu_type=GPU_TYPE,
        region=REGION,
        git_url=REPO_URL,
        commit_sha=COMMIT_SHA,
        image_tag=IMAGE_TAG,
        image_url='',
        options=OPTIONS,
    )

    GoogleCloudPlatform.train.assert_called_once_with()

    assert result.exit_code == 0


def test_train_yaml(mocker):
    mocker.patch(
        "hydra.cli.os.path.isfile",
        return_value=True
    )
    mocker.patch(
        "hydra.cli.check_repo",
        return_value=(REPO_URL, COMMIT_SHA)
    )
    mocker.patch(
        'hydra.cli.LocalPlatform.__init__',
        return_value=None
    )
    mocker.patch(
        'hydra.cli.LocalPlatform.train',
    )

    runner = CliRunner()
    result = runner.invoke(train,
        ['-y', TEST_YAML_PATH,
        '--github_token', GITHUB_TOKEN])

    LocalPlatform.__init__.assert_has_calls(
        [
            mocker.call(
                model_path=MODEL_PATH,
                options='epoch=88 lr=0.01',
                git_url=REPO_URL,
                commit_sha=COMMIT_SHA,
                github_token=GITHUB_TOKEN,
            ),
            mocker.call(
                model_path=MODEL_PATH,
                options='epoch=88 lr=0.1',
                git_url=REPO_URL,
                commit_sha=COMMIT_SHA,
                github_token=GITHUB_TOKEN,
            ),
            mocker.call(
                model_path=MODEL_PATH,
                options='epoch=88 lr=1',
                git_url=REPO_URL,
                commit_sha=COMMIT_SHA,
                github_token=GITHUB_TOKEN,
            ),
            mocker.call(
                model_path=MODEL_PATH,
                options='epoch=888 lr=0.1',
                git_url=REPO_URL,
                commit_sha=COMMIT_SHA,
                github_token=GITHUB_TOKEN,
            ),
            mocker.call(
                model_path=MODEL_PATH,
                options='epoch=888 lr=1',
                git_url=REPO_URL,
                commit_sha=COMMIT_SHA,
                github_token=GITHUB_TOKEN,
            )
        ],
        any_order=True
    )



    LocalPlatform.train.assert_called_with()

    assert result.exit_code == 0



import pytest
import warnings
import pytest_mock
from hydra.utils.git_repo import *

VALID_MULTIPLE_FILES = ["shopify.inc", "clickup.tm"]
VALID_MULTIPLE_COMMITS = ["m1rr0r1ng"]
VALID_BRANCH_NAME = "bay3s1an"

def test_GitRepo_is_empty_true(mocker):
    repo = mocker.Mock()
    repo.bare = True

    git_repo = GitRepo(repo)
    result = git_repo.is_empty()

    assert result == True


def test_GitRepo_is_empty_false(mocker):
    repo = mocker.Mock()
    repo.bare = False

    git_repo = GitRepo(repo)
    result = git_repo.is_empty()

    assert result == False


def test_GitRepo_is_untracked_true(mocker):
    repo = mocker.Mock()
    repo.untracked_files = VALID_MULTIPLE_FILES

    git_repo = GitRepo(repo)
    result = git_repo.is_untracked()

    assert result == True


def test_GitRepo_is_untracked_false(mocker):
    repo = mocker.Mock()
    repo.untracked_files = []

    git_repo = GitRepo(repo)
    result = git_repo.is_untracked()

    assert result == False


def test_GitRepo_is_modified_true(mocker):
    repo = mocker.Mock()
    repo.index.diff.return_value = VALID_MULTIPLE_COMMITS

    git_repo = GitRepo(repo)
    result = git_repo.is_modified()

    repo.index.diff.assert_called_once_with(None)
    assert result == True


def test_GitRepo_is_modified_false(mocker):
    repo = mocker.Mock()
    repo.index.diff.return_value = []

    git_repo = GitRepo(repo)
    result = git_repo.is_modified()

    repo.index.diff.assert_called_once_with(None)
    assert result == False


def test_GitRepo_is_uncommitted_true(mocker):
    repo = mocker.Mock()
    repo.index.diff.return_value = VALID_MULTIPLE_COMMITS

    git_repo = GitRepo(repo)
    result = git_repo.is_uncommitted()

    repo.index.diff.assert_called_once_with("HEAD")
    assert result == True


def test_GitRepo_is_uncommitted_false(mocker):
    repo = mocker.Mock()
    repo.index.diff.return_value = []

    git_repo = GitRepo(repo)
    result = git_repo.is_uncommitted()

    repo.index.diff.assert_called_once_with("HEAD")
    assert result == False


def test_GitRepo_is_unsynced_true(mocker):
    repo = mocker.Mock()
    repo.active_branch.name = VALID_BRANCH_NAME
    repo.iter_commits.return_value = VALID_MULTIPLE_COMMITS

    git_repo = GitRepo(repo)
    result = git_repo.is_unsynced()

    repo.iter_commits.assert_called_once_with('origin/{}..{}'.format(VALID_BRANCH_NAME, VALID_BRANCH_NAME))
    assert result == True


def test_GitRepo_is_unsynced_false(mocker):
    repo = mocker.Mock()
    repo.active_branch.name = VALID_BRANCH_NAME
    repo.iter_commits.return_value = []

    git_repo = GitRepo(repo)
    result = git_repo.is_unsynced()

    repo.iter_commits.assert_called_once_with('origin/{}..{}'.format(VALID_BRANCH_NAME, VALID_BRANCH_NAME))
    assert result == False



class GitRepo():
    def __init__(self, repo):
        self.repo = repo

    def is_empty(self):
        return self.repo.bare

    def is_untracked(self):
        return len(self.repo.untracked_files) > 0

    def is_modified(self):
        return len(self.repo.index.diff(None)) > 0

    def is_uncommitted(self):
        return len(self.repo.index.diff("HEAD")) > 0

    def is_unsynced(self):
        branch_name = self.repo.active_branch.name
        count_unpushed_commits = len(list(self.repo.iter_commits('origin/{}..{}'.format(branch_name, branch_name))))
        return count_unpushed_commits > 0



import re
import os
import git
import warnings
import subprocess
from hydra.utils.git_repo import GitRepo


def get_repo_url():
    git_url = subprocess.check_output("git config --get remote.origin.url", shell=True).decode("utf-8").strip()
    git_url = re.compile(r"https?://(www\.)?").sub("", git_url).strip().strip('/')
    if len(git_url.split(":")) > 1:
        git_url = git_url.split(":")[-1]
        git_url = f"github.com/{git_url}"
    return git_url


def get_commit_sha():
    commit_sha = subprocess.check_output("git log --pretty=tformat:'%h' -n1 .", shell=True).decode("utf-8").strip()
    return commit_sha


def check_repo(github_token, repo=None):
    if github_token == None:
        raise Exception("GITHUB_TOKEN not found in environment variable or as argument.")

    if repo is None:
        repo = git.Repo(os.getcwd())
        repo = GitRepo(repo)

    if repo.is_empty():
        raise Exception("Hydra is not being called in the root of a git repo.")

    if repo.is_untracked():
        warnings.warn("Some files are not tracked by git.", UserWarning)

    if repo.is_modified():
        raise Exception("Some modified files are not staged for commit.")

    if repo.is_uncommitted():
        raise Exception("Some staged files are not commited.")

    if repo.is_unsynced():
        raise Exception("Some commits are not pushed to the remote repo.")

    repo_url = get_repo_url()
    commit_sha = get_commit_sha()

    return repo_url, commit_sha



import os
import yaml
import json
import click
import hydra.utils.constants as const
from hydra.utils.git import check_repo
from hydra.utils.utils import dict_to_string, inflate_options
from hydra.cloud.local_platform import LocalPlatform
from hydra.cloud.fast_local_platform import FastLocalPlatform
from hydra.cloud.google_cloud_platform import GoogleCloudPlatform
from hydra.cloud.aws_platform import AWSPlatform

from hydra.version import __version__

@click.group()
@click.version_option(__version__)
def cli():
    pass


@cli.command()
# Generic options
@click.option('-y', '--yaml_path', default='hydra.yaml', type=str)
@click.option('-p', '--project_name', default=None, type=str)
@click.option('-m', '--model_path', default=None, type=str)
@click.option('--cloud', default=None, type=click.Choice(['fast_local','local', 'aws', 'gcp', 'azure'], case_sensitive=False))
@click.option('--github_token', envvar='GITHUB_TOKEN') # Takes either an option or environment var

# Cloud specific options
@click.option('--cpu_count', default=None, type=click.IntRange(0, 96), help='Number of CPU cores required')
@click.option('--memory_size', default=None, type=click.IntRange(0, 624), help='GB of RAM required')

@click.option('--gpu_count', default=None, type=click.IntRange(0, 8), help="Number of accelerator GPUs")
@click.option('--gpu_type', default=None, type=str, help="Accelerator GPU type")

@click.option('--region', default=None, type=str, help="Region of cloud server location")

# Docker Options
@click.option('-t', '--image_tag', default=None, type=str, help="Docker image tag name")
@click.option('-u', '--image_url', default=None, type=str, help="Url to the docker image on cloud")

# Env variable of model file
@click.option('-o', '--options', default=None, type=str, help='Environmental variables for the script')

def train(
    yaml_path,
    project_name,
    model_path,
    cloud,
    github_token,
    cpu_count,
    memory_size,
    gpu_count,
    gpu_type,
    region,
    image_tag,
    image_url,
    options):

    # If YAML config file available to supplement the command line arguments
    if os.path.isfile(yaml_path):
        with open(yaml_path) as f:
            print("[Hydra Info]: Loading run info from {}...".format(yaml_path))

            data = yaml.load(f, Loader=yaml.FullLoader)
            train_data = data.get('train', '')
            project_name = train_data.get('project_name')

            if project_name is None:
                raise Exception("project_name option is required")

            model_path = train_data.get('model_path', const.MODEL_PATH_DEFAULT) if model_path is None else model_path
            cloud = train_data.get('cloud', const.CLOUD_DEFAULT).lower() if cloud is None else cloud
            if cloud == 'gcp' or cloud == 'aws':
                region = train_data.get('region', const.REGION_DEFAULT) if region is None else region

                cpu_count = train_data.get('cpu_count', const.CPU_COUNT_DEFAULT) if cpu_count is None else cpu_count
                memory_size = train_data.get('memory_size', const.MEMORY_SIZE_DEFAULT) if memory_size is None else memory_size
                gpu_count = train_data.get('gpu_count', const.GPU_COUNT_DEFAULT) if gpu_count is None else gpu_count
                gpu_type = train_data.get('gpu_type', const.GPU_TYPE_DEFAULT) if gpu_type is None else gpu_type

                image_tag = train_data.get('image_tag', const.IMAGE_TAG_DEFAULT) if image_tag is None else image_tag
                image_url = train_data.get('image_url', const.IMAGE_URL_DEFAULT) if image_url is None else image_url

            elif cloud == 'local' or cloud == 'fast_local':
                pass

            else:
                raise Exception("Reached parts of Hydra that are either not implemented or recognized.")

            options_list = train_data.get('options', const.OPTIONS_DEFAULT) if options is None else options
            if type(options_list) is str:
                options_list = json.loads(options_list)
    # Read the options for run from CIL
    else:
        model_path = const.MODEL_PATH_DEFAULT if model_path is None else model_path
        cloud = const.CLOUD_DEFAULT if cloud is None else cloud

        region = const.REGION_DEFAULT if region is None else region

        cpu_count = const.CPU_COUNT_DEFAULT if cpu_count is None else cpu_count
        memory_size = const.MEMORY_SIZE_DEFAULT if memory_size is None else memory_size
        gpu_count = const.GPU_COUNT_DEFAULT if gpu_count is None else gpu_count
        gpu_type = const.GPU_TYPE_DEFAULT if gpu_type is None else gpu_type

        image_tag = const.IMAGE_TAG_DEFAULT if image_tag is None else image_tag
        image_url = const.IMAGE_URL_DEFAULT if image_url is None else image_url

        options = str(const.OPTIONS_DEFAULT) if options is None else options
        options_list = json.loads(options)

    if isinstance(options_list, dict):
        options_list = [options_list]

    options_list_inflated = inflate_options(options_list)
    git_url, commit_sha = check_repo(github_token)

    hydra_core_configs = {
        'HYDRA_PLATFORM': cloud,
        'HYDRA_GIT_URL': git_url,
        'HYDRA_COMMIT_SHA': commit_sha,
        'HYDRA_OAUTH_TOKEN': github_token,
        'HYDRA_MODEL_PATH': model_path
    }

    print("\n[Hydra Info]: Executing experiments with the following options: \n {}\n".format(options_list_inflated))

    for i, options in enumerate(options_list_inflated):
        options_str = dict_to_string(options)
        hydra_core_configs_str = dict_to_string(hydra_core_configs)

        print("\n[Hydra Info]: Runnning experiment #{} with the following options: \n {}\n".format(i, options))

        if cloud == 'fast_local':
            platform = FastLocalPlatform(model_path,
                                         f"{options_str} {hydra_core_configs_str}")
            platform.train()
            continue

        if cloud == 'local':
            platform = LocalPlatform(
                model_path=model_path,
                options=options_str,
                git_url=git_url,
                commit_sha=commit_sha,
                github_token=github_token)

        elif cloud == 'gcp':
            platform = GoogleCloudPlatform(
                model_path=model_path,
                github_token=github_token,
                cpu=cpu_count,
                memory=memory_size,
                gpu_count=gpu_count,
                gpu_type=gpu_type,
                region=region,
                git_url=git_url,
                commit_sha=commit_sha,
                image_url=image_url,
                image_tag=image_tag,
                options=options_str)

        elif cloud == 'aws':
            platform = AWSPlatform(
                model_path=model_path,
                project_name=project_name,
                github_token=github_token,
                cpu=cpu_count,
                memory=memory_size,
                gpu_count=gpu_count,
                region=region,
                git_url=git_url,
                commit_sha=commit_sha,
                image_url=image_url,
                image_tag=image_tag,
                options=options
            )

        else:
            raise Exception("Reached parts of Hydra that are not yet implemented.")

        platform.train()

    return 0



import os
from google.cloud import storage

def get_gcp_dataset(project_name, bucket_name, source_path, destination_path):
    # project_name = "your-project-name"
    # bucket_name = "your-bucket-name"
    # source_path = "storage-object-name"
    # destination_filepath = "file-path"
    if os.path.isfile(destination_path):
        print("[PROJECT INFO] {} already exists, skipping download...".format(destination_path))
        return 0

    client = storage.Client(project_name)
    bucket = client.get_bucket(bucket_name)

    bucket.get_blob(source_path).download_to_filename(destination_path)

    print("[PROJECT INFO] Blob {} downloaded to {}.".format(source_path, destination_path))

    return 0


def save_gcp_result(project_name, bucket_name, source_path, destination_path):
    client = storage.Client(project_name)
    bucket = client.get_bucket(bucket_name)

    bucket.blob(destination_path).upload_from_filename(filename=source_path)

    print("[PROJECT INFO] {} saved to {}.".format(source_path, destination_path))

    return 0



import setuptools
from hydra.version import __version__


f = open('requirements.txt')
installation_requirements = f.read().split()

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hydra-ml",
    version=__version__,
    author="Hydra Development Team",
    author_email="faisal.anees@georgian.io",
    description="A cloud-agnostic ML Platform",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/georgianpartners/hydra",
    py_modules=['hydra'],
    install_requires=installation_requirements,
    entry_points='''
        [console_scripts]
        hydra=hydra.cli:cli
    ''',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Artificial Intelligence"
    ],
    python_requires='>=3.6',
)



import os
import subprocess
from hydra.cloud.abstract_platform import AbstractPlatform

class LocalPlatform(AbstractPlatform):
    def __init__(self, model_path, options, git_url, commit_sha, github_token):
        self.git_url = git_url
        self.commit_sha = commit_sha
        self.github_token = github_token

        self.script_path = os.path.join(os.path.dirname(__file__), '../docker/local_execution.sh')

        super().__init__(model_path, options)

    def train(self):
        command = ['sh', self.script_path, '-g', self.git_url, '-c', self.commit_sha,
            '-o', self.github_token, '-m', self.model_path, '-p', self.options]

        self.run_command(command)
        return 0

    def serve(self):
        pass



import subprocess

class AbstractPlatform():
    def __init__(self, model_path, options):
        self.model_path = model_path
        self.options = options

    def train(self):
        raise Exception("Not Implemented: Please implement this function in the subclass.")

    def serve(self):
        raise Exception("Not Implemented: Please implement this function in the subclass.")

    def run_command(self, command):
        subprocess.run(command)
